/*
 * Copyright (C) 2007 BEYLER Jean Christophe
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include "HandleImage.h"

/* macros =================================================================== */
#define NBR_TYPE (sizeof(animations)/(sizeof(animations[0])))

/* constants ================================================================ */
/* types ==================================================================== */
typedef struct sAnimFct_ SAnimFct;

/* structures =============================================================== */
struct sAnimFct_
{
  void (*init) (SImage*);
  int (*isfinished) (SImage*);
  void (*frame) (SImage*, SDL_Surface*);
};

/* private variables ======================================================== */
static SAnimFct animations[] =
  {
    {Image_FadeIn_Init, Image_FadeIn_IsFinished, Image_FadeIn_Frame},
    {Image_FadeOut_Init, Image_FadeOut_IsFinished, Image_FadeOut_Frame},
    {Image_FadeIn_Init, Image_FadeIn_IsFinished, Image_FadeInMove_Frame},
    {Image_FadeOut_Init, Image_FadeOut_IsFinished, Image_FadeOutMove_Frame}
  };

/* private functions ======================================================== */
/*
 * Type aleatoire
 * Random type retourne un type d'animation qui sera utilise
 */
static int Handle_RandomType (void)
{
  return rand() % NBR_TYPE;
}

/*
 * New type choisit un nouveau type pour l'image
 */
static void Handle_NewType (SHandleImage *hi, SListImage *elem)
{
  /* Verifications */
  if ( (elem != NULL) && (hi != NULL) )
  {
    /* Position aleatoire */
    Image_RandomPos (elem->im, & (hi->area) );
    /* Nouveau type */
    elem->type_anim = Handle_RandomType();
    /* Appel de la fonction d'initialisation de ce type d'animation */
    animations[elem->type_anim].init (elem->im);
  }
}

/* internal public functions ================================================ */
/* entry points ============================================================= */
/*
 * Fonction d'initialisation
 */
SHandleImage *Handle_Init (const char *fname, SDL_Rect *area)
{
  SHandleImage *res;
  FILE *f;

  /* Si le nom est NULL */
  if (fname == NULL)
  {
    return NULL;
  }

  /* Si la zone est NULL */
  if (area == NULL)
  {
    return NULL;
  }

  /* On teste l'ouverture du fichier */
  f = fopen (fname, "rb");
  if (f == NULL)
  {
    return NULL;
  }

  /* On alloue */
  res = malloc (sizeof (*res) );

  /* Verification */
  if (res != NULL)
  {
    res->list = NULL;
    res->area = *area;

    /* Allocation pour le nom */
    res->fname = malloc (strlen (fname) + 1);
    strcpy (res->fname, fname);
  }

  /* Retour */
  return res;
}

/*
 * Destruction
 */
void Handle_Destroy (SHandleImage *hi)
{
  if (hi != NULL)
  {
    ListImage_DestroyList (hi->list);
    free (hi->fname), hi->fname = NULL;
  }
}

/*
 * Ajout d'une image : retourne 1 si tout c'est bien passe, 0 sinon
 */
int Handle_AddImage (SHandleImage *hi)
{
  /* Verification */
  if (hi != NULL)
  {
    /* Creation de l'image */
    SImage *im = Image_Create (hi->fname);

    /* Verification */
    if (im != NULL)
    {
      /* Creation d'un element */
      SListImage *elem = ListImage_Create();

      /* Verification */
      if (elem != NULL)
      {
        /* Position aleatoire */
        Image_RandomPos (im, &hi->area);

        /* On remplit la structure elem */
        elem->im = im;

        /* On met a jour le next */
        elem->next = hi->list;

        /* Nouveau type d'animation */
        elem->type_anim = Handle_RandomType();

        /* On met a jour la liste d'objet */
        hi->list = elem;

        /* Tout s'est bien passe */
        return 1;
      }
      else
      {
        /* Elem == NULL, on detruit l'image */
        Image_Destroy (im);
      }

    }
  }
  return 0;
}

/*
 * On enleve une image
 */
int Handle_RemoveImage (SHandleImage *hi)
{
  /* Verifications */
  if (hi != NULL)
  {
    if (hi->list != NULL)
    {
      /* On va enlever le premier element */
      SListImage *elem = hi->list->next;

      ListImage_DestroyElem (hi->list);

      hi->list = elem;

      /* Tout s'est bien passe */
      return 1;
    }
  }
  /* Soit hi == NULL, soit hi->list == NULL */
  return 0;
}

/*
 * Fonction de rendu
 */
void Handle_Frame (SHandleImage *hi, SDL_Surface *screen)
{
  if (hi != NULL)
  {
    SListImage *elem = hi->list;

    /* On parcourt la liste */
    while (elem != NULL)
    {
      SImage *im = elem->im;

      /* Est-ce que l'animation est finie ? */
      if (animations[elem->type_anim].isfinished (im) )
      {
        /* Alors on demande un nouveau type pour cette image */
        Handle_NewType (hi, elem);
      }
      else
      {
        /* On appelle la fonction de rendu de ce type d'animation */
        animations[elem->type_anim].frame (im, screen);
      }

      /* Prochain element */
      elem = elem->next;
    }
  }

}

/* public variables ========================================================= */
